"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _easevalidation = require("easevalidation");

var _pluralize = _interopRequireDefault(require("pluralize"));

var _oors = require("oors");

var _camelCase = _interopRequireDefault(require("lodash/camelCase"));

var _merge = _interopRequireDefault(require("lodash/merge"));

var _path = _interopRequireDefault(require("path"));

var _graphql = require("oors-mongodb/build/graphql");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class RADModule extends _oors.Module {
  constructor(...args) {
    super(...args);
    this.name = 'oors.rad';
    this.services = {};
    this.hooks = {
      'oors.graphql.buildContext': ({
        context
      }) => {
        Object.assign(context, {
          getService: this.getService,
          getModule: name => this.manager.get(name)
        });
      }
    };

    this.collectFromModule = async module => {
      const config = (0, _merge.default)({}, this.getConfig('moduleDefaultConfig'), module.getConfig(this.name));
      const tasks = [];
      const wrapper = this.deps['oors.autoloader'].wrap(module);

      if (config.autoload.services) {
        tasks.push(this.loadModuleServices(wrapper));
      }

      if (config.autoload.methods) {
        tasks.push(this.loadModuleMethods(wrapper));
      }

      if (!tasks.length) {
        return;
      }

      await Promise.all(tasks);
    };

    this.setService = (key, service) => {
      this.services[key] = Object.assign(service, {
        getService: this.getService
      });
    };

    this.getService = key => this.services[key];

    this.getLoadersName = repositoryName => (0, _pluralize.default)((0, _camelCase.default)(repositoryName));
  }

  async setup({
    autoCreateLoaders
  }) {
    const dependencies = ['oors.autoloader'];

    if (autoCreateLoaders) {
      dependencies.push('oors.graphql', 'oors.mongodb');
    }

    await this.loadDependencies(dependencies);
    await this.runHook('load', this.collectFromModule);
    this.exportProperties(['registerModuleService', 'setService', 'getService', 'getLoadersName']);

    if (autoCreateLoaders) {
      const {
        loaders
      } = this.deps['oors.graphql'];
      const {
        repositories
      } = this.deps['oors.mongodb']; // creating loaders for repositories that have been added already

      Object.keys(repositories).forEach(repositoryName => {
        this.deps['oors.graphql'].addLoaders((0, _graphql.createLoaders)(repositories[repositoryName]), this.getLoadersName(repositoryName));
      });
      this.onModule('oors.mongodb', 'repository', ({
        repository,
        key
      }) => {
        this.deps['oors.graphql'].addLoaders((0, _graphql.createLoaders)(repository), this.getLoadersName(key));
      });
      this.export({
        getLoaders: repositoryName => loaders[this.getLoadersName(repositoryName)]
      });
    }
  }

  async loadModuleServices({
    module,
    glob
  }) {
    const files = await glob('services/*.js', {
      nodir: true
    });
    files.forEach(file => {
      const Service = require(file).default; // eslint-disable-line global-require, import/no-dynamic-require


      const service = new Service(module);

      if (!service.name) {
        throw new Error(`Unable to register a service without a name! "${file}" in "${module.name}" module`);
      }

      this.registerModuleService(module, service);
    });
  } // eslint-disable-next-line class-methods-use-this


  async loadModuleMethods({
    module,
    glob
  }) {
    const files = await glob('methods/*.js', {
      nodir: true
    });
    files.forEach(file => {
      const method = require(file).default; // eslint-disable-line global-require, import/no-dynamic-require


      const {
        name
      } = _path.default.parse(file);

      if (typeof method !== 'function') {
        throw new Error(`Unable to register "${name}" method for ${module.name} module! (not a function)`);
      }

      module.export(name, (...args) => method.call(module, {
        args,
        module,
        getService: this.getService
      }));
    });
  }

  registerModuleService(module, service) {
    this.setService(`${module.name}.${service.name}`, service);
  }

}

RADModule.validateConfig = (0, _easevalidation.validate)(_easevalidation.validators.isSchema({
  autoCreateLoaders: [_easevalidation.validators.isDefault(true), _easevalidation.validators.isBoolean()],
  moduleDefaultConfig: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isSchema({
    autoload: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isSchema({
      services: [_easevalidation.validators.isDefault(true), _easevalidation.validators.isBoolean()],
      methods: [_easevalidation.validators.isDefault(true), _easevalidation.validators.isBoolean()]
    })]
  })]
}));
var _default = RADModule;
exports.default = _default;